package com.persistent.wizard;

/**
 * User: Yuvraj Sawant.
 * Date: September 3, 2006
 */

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Used to create a page to be added to a wizard.
 */
public abstract class JPage extends JPanel{

	
	private String previousPageName;
	private String nextPageName;
	JFrame frame;
	JDialog dialog;
	JPanel pagePanel;
	private JPanel wizardPanel= new JPanel();
	private JButton back = new JButton("Back");
	private JButton finish = new JButton("Finish");
	private JButton next = new JButton("Next");
	private JButton cancel = new JButton("Cancel");
	GridBagConstraints gridConstraints=new GridBagConstraints();

    private JWizard wizard;
	public JPage()
	{
		super();
		setLayout(new BorderLayout());
		createPage();
	}

    /**
     * To be implimented to enable or disaple the next button.
     * @return boolean.
     */
    protected abstract boolean canFlipToNextPage();
   

    /**
     * Creates a JPanel to be displayed in wizard.
     * @return JPanel
     */
    protected abstract JPanel createContentPanel();
	public abstract String setTitle();
	protected abstract String setDescriptionTitle();
	protected abstract String setDescription();

    /**
     *  To be called to create a page with browsing buttons and the user provided JPanel.
     */
    private void createPage()
	{
		super.add(createDescriptionPanel(),BorderLayout.NORTH);
		JPanel userPanel = createContentPanel();
		userPanel.setBorder(new TitledBorder(""));
		super.add(userPanel,BorderLayout.CENTER);
		super.add(createBrowseButtons(),BorderLayout.SOUTH);
	}


    /*
     * Create a panel to display description of the page.
     */
    private JPanel createDescriptionPanel()
	{
		JTextArea panelName = new JTextArea(1, 30);
		Font headingFont = new Font("Arial",Font.BOLD , 16);
		Font descFont = new Font("Arial", Font.ITALIC, 12);
		panelName.setFont(headingFont);
        panelName.setBackground(Color.WHITE);
        panelName.setEditable(false);
		JTextArea panelDescription = new JTextArea(3, 10);
		panelDescription.setWrapStyleWord(true);
		panelDescription.setFont(descFont);
        panelDescription.setBackground(Color.WHITE);
        panelDescription.setEditable(false);

		panelName.setText(setDescriptionTitle());
		panelDescription.setText(setDescription());
		FlowLayout descriptionLayout = new FlowLayout();
		descriptionLayout.setAlignment(FlowLayout.LEFT);
		JPanel descriptionPanel = new JPanel(descriptionLayout);
		descriptionPanel.add(panelName,BorderLayout.NORTH);
		descriptionPanel.add(panelDescription,BorderLayout.CENTER);
		descriptionPanel.setBackground(Color.WHITE);
		return descriptionPanel;
	}
	
    /*
     * Create the browser buttons Panel.
     */
	private JPanel createBrowseButtons()
	{
		FlowLayout buttonLayout = new FlowLayout();
		buttonLayout.setAlignment(FlowLayout.RIGHT);
		JPanel buttonPanel = new JPanel(buttonLayout);
		buttonPanel.add(back);
		buttonPanel.add(finish);
		buttonPanel.add(next);
		buttonPanel.add(cancel);
		
		
		back.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {

			 callPreviousPage();	
			}
			
		});
		
		finish.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {

				callFinish();
			}
			
		});
		
		next.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {

				callNextPage();
			}
			
		});
		
		cancel.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {

				callClose();
			}
			
		});
		
		return buttonPanel;
	}
	
	/*
	 * On click of back button we have to load the previous page.
	 */
	private void callPreviousPage()
	{
		CardLayout card  = (CardLayout)wizardPanel.getLayout() ;
		card.show(wizardPanel,previousPageName);
		wizard.presentPage = previousPageName;
		JPage displayedPage = wizard.getPresentPage();
        displayedPage.flipToNextPage();
        displayedPage.setFinishEnabled();
		
	}
	
	/*
	 * On click of finish button call the implemented performFinish API which specifies the job to be done on click of finish.
	 */
	private void callFinish()
	{
		wizard.performFinish();
        dialog.dispose();
		frame.dispose();
    }
	/*
	 * On press of next button call the next page to be displayed.
	 */
	private void callNextPage()
	{
		CardLayout card  = (CardLayout)wizardPanel.getLayout() ;
        card.show(wizardPanel,nextPageName);
        wizard.presentPage = nextPageName;
        JPage displayedPage = wizard.getPresentPage();
        displayedPage.flipToNextPage();
        displayedPage.setFinishEnabled();
        
    }
	/*
	 * close the wizard.
	 */
	private void callClose()
	{
        wizard.performCancel();
        dialog.dispose();
		frame.dispose();
	}
	
	/**
	 * This is to set the next page name for each page. This checks if user has overridden getUsersNextPageName() API and specified next 
	 * page name then that page is considered as next page else next consecutive page is considered as next page.
	 * @param nextPage next page name to set.
	 */
	final public void setNextPageName(String nextPage)
	{
        String nextPageNameFromUser = getUsersNextPageName();
        if(nextPageNameFromUser == null)
            nextPageName = nextPage;
        else
        {
            nextPageName = nextPageNameFromUser;
        }
    }

    /**
     * Returns the name of the next page which is going to be displayed.
     * @return String to get the next page name.
     */
    final public String getNextPageName()
    {
        return nextPageName;
    }

    /**
     *  The function to be overridden when the next page to be used is userSpecified.
     * @return String if you have a fixed next page you can put it's name here.
     */

    protected String getUsersNextPageName()
    {
        return null;
    }

    /**
     * This is used set the previous page name.This is used internally. Not to be used by user of this architecture.
     * @param previousPage name of previous page to set.
     */
    final public void setPreviousPageName(String previousPage)
	{
        String previousPageNameFromUser = getUsersPreviousPageName();
        if(previousPageNameFromUser == null)
            previousPageName = previousPage;
        else
        {
            previousPageName = previousPageNameFromUser;
        }
	}

    /**
     *  This is used to get user specified previous page name. User has to overide this method to specify the previous page name he wants to be displayed.
     * @return String name of previous page.
     */
    protected String getUsersPreviousPageName()
    {
        return null;
    }

    /**
     *  Set the wizard panel to which the present page is attached to. This is used internally. Not to be used by user of this architecture.
     * @param wizardPanel
     */
    final public void setWizardPanel(JPanel wizardPanel)
	{
	this.wizardPanel = wizardPanel;	
	}

    /**
     *  This is the dialog which is created to display a wizard.This is used internally. Not to be used by user of this architecture.
     * @param wizardDialog
     */
    final public void setWizard(JWizard wizardDialog)
	{
	this.wizard = wizardDialog;	
	}

    /**
     * To enable the next button so that one can move to next page.
     */
    final public void flipToNextPage()
	{
		next.setEnabled(canFlipToNextPage());
	}

    
    /**
     * This is to enable finish usinfg the implementation of the canFinish function provided by user.This is used internally. Not to be used by user of this architecture.
     */
    final public void setFinishEnabled()
	{
    	
		finish.setEnabled(wizard.canFinish());
	}

    /**
     * This is to enable the back  button.If  any previous page is available it is enabled.
     */
    final public void flipToPreviousPage()
	{
		back.setEnabled(canGoToPreviousPage());
	}

    private boolean canGoToPreviousPage()
	{
        return previousPageName != null;
	}

    /**
     * This has to be used to check out wether it is a valid situation to enable finish,next or previous button.
     */
    protected final void setPageComplete()
	{
		setFinishEnabled();
        flipToNextPage();
        setPreviousPageName(previousPageName);
        setNextPageName(nextPageName);
    }
    
}
