package com.persistent.wizard;

/**
 * User: Yuvraj Sawant.
 * Date: September 3, 2006
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * This is to Create  a Dialog to show the wizard.
 */
public abstract class JWizard extends JWizardPageTraversal{
	private JFrame frame;
	private JPanel wizardPanel;
    private JDialog wizardDialog;
	private String wizardName;
	/**
	 * 
	 * @param frame any JFrame object with whatever properties you want to set for this frame.
	 * @param wizardName Name of the wizard. This is displayed as the frame/dialog name.
	 */
	public JWizard(final JFrame frame,String wizardName)
	{
		this.frame = frame; 
		this.wizardName = wizardName;
        WindowListener wndCloser = new WindowAdapter()
               {
                 public void windowClosing(WindowEvent e)
                 {
                   wizardDialog.dispose();
                   frame.dispose();
                 }
               };
               frame.addWindowListener(wndCloser);
        frame.setResizable(false);
        createWizard();
        
	}
	
	/**
	 * This is used to create default frame whose size dimentions are 600px X 600 px.
	 * @param wizardName Name of the wizard. This is displayed as the frame/dialog name.
	 */
	public JWizard(String wizardName)
	{
		this.frame = new JFrame(); 
		this.frame.setSize(600,600);
		this.wizardName = wizardName;
        WindowListener wndCloser = new WindowAdapter()
               {
                 public void windowClosing(WindowEvent e)
                 {
                   wizardDialog.dispose();
                   frame.dispose();
                 }
               };
               frame.addWindowListener(wndCloser);
        frame.setResizable(false);
        createWizard();
	}

	/**
	 * Creates default JFrame object but allows user to set dimentions of Wizard.
	 * @param wizardName Name of the wizard. This is displayed as the frame/dialog name.
	 * @param dimension Dimentions of the wizard. eg(600pixel X 600pixel)
	 */
	public JWizard(String wizardName,Dimension dimension)
	{
		this.frame = new JFrame(); 
		frame.setSize(dimension);
		this.wizardName = wizardName;
        WindowListener wndCloser = new WindowAdapter()
               {
                 public void windowClosing(WindowEvent e)
                 {
                   wizardDialog.dispose();
                   frame.dispose();
                 }
               };
               frame.addWindowListener(wndCloser);
        frame.setResizable(false);
        createWizard();
	}

	/**
	 * User is allowed to specify his dimentions to resize Wizard.
	 * @param frame JFrame object on which you want this wizard to be shown.
	 * @param wizardName Name of the wizard. This is displayed as the frame/dialog name.
	 * @param dimension Dimentions of the wizard. eg(600pixel X 600pixel)
	 */
	public JWizard(final JFrame frame,String wizardName,Dimension dimension)
	{
		this.frame = frame; 
		frame.setSize(dimension);
		this.wizardName = wizardName;
        WindowListener wndCloser = new WindowAdapter()
               {
                 public void windowClosing(WindowEvent e)
                 {
                   wizardDialog.dispose();
                   frame.dispose();
                 }
               };
               frame.addWindowListener(wndCloser);
        frame.setResizable(false);
        createWizard();
	}
	
	
	private void createWizard()
	{
		 setThePositionToCenter(frame);
		 createWizardPanel();
		 frame.add(wizardPanel);
		 wizardDialog = new JDialog(frame, wizardName, true);
         wizardDialog.setResizable(false);
		 wizardDialog.setContentPane(wizardPanel);
		 wizardDialog.setSize(frame.getSize());
		 wizardDialog.setLocation(frame.getLocation());
		 
	}
	
	/**
     * This is to display the  dialog / frame at center.
     */
    private void setThePositionToCenter(Window window)
    {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension dimension = toolkit.getScreenSize();

        Double screenWidth = new Double(dimension.getWidth() / 2);
        Double screenHeight = new Double(dimension.getHeight() / 2);

        int locationX = screenWidth.intValue() - (window.getWidth() / 2);
        int locationY = screenHeight.intValue() - (window.getHeight() / 2);
        window.setLocation(locationX, locationY);
    }
    
	/**
	 * Once you have added all pages you want to display to the wizard you have to call this to make the wizard visible.
	 *
	 */
	public void showDialog()
	{
		WindowListener wndCloser = new WindowAdapter()
	    {
	      public void windowClosing(WindowEvent e) 
	      {
	    	  closeWizard();
	      }
	    };
		wizardDialog.addWindowListener(wndCloser);
		wizardDialog.setVisible(true);
	}
	
	private void closeWizard()
	{
        performCancel();
        wizardDialog.dispose();
  	  	frame.dispose();
	}


/**
 * This is to add pages to wizard. Pages are called in sequence if you have not set next page name or not overridden getUsersNextPageName() API in his JPage implementation. 
 * @param pageToAdd This is object of class extending JPage. This is a page to add to wizard. 
 * @param name provide a name to a page to be added.
 * @return true if page is added successfully and false if fails to add a page.
 */
    public boolean addPage(JPage pageToAdd,String name)
	{
    	boolean isFirstPage=false;
		if(presentPage== null || presentPage.equals(""))
		{
			presentPage = name;
			isFirstPage=true;
		}
		boolean returnValue  = super.add(pageToAdd,name);
		pageToAdd.setWizardPanel(wizardPanel);
		pageToAdd.frame = frame;
		pageToAdd.dialog = wizardDialog;
		pageToAdd.setWizard(this);
		
		if(!returnValue)
			return false;
		else
		{
			JPage page =getPageByName(name);
			wizardPanel.add(page,name);
			if(isFirstPage)
			{	
				pageToAdd.setFinishEnabled();
				pageToAdd.flipToNextPage();
				pageToAdd.flipToPreviousPage();
			}
			return true;
		}
	}
	
	private void createWizardPanel()
	{
		CardLayout layout = new CardLayout();
		wizardPanel = new JPanel(layout);
		
		
	}
	
}

