package com.persistent.wizard;
/**
 * User: Yuvraj Sawant.
 * Date: September 3, 2006
 */

import java.util.LinkedHashMap;

/**
 * IT generates a wrapper over the JWizard to traverse the wizard
 */
abstract class JWizardPageTraversal implements WizardFunction{
	
	
	private int pageCount=-1;
	private Integer presentPageNumber = new Integer(0);
	private LinkedHashMap pageswithName = new LinkedHashMap();
	private LinkedHashMap pagesWithNumber = new LinkedHashMap();
	public String presentPage;

    /**
     * returns the Next page to be displayed.
     * @return JPage
     */
    public JPage getNextPage() {
		if(presentPageNumber.intValue()>=pageCount)
		{
			//There is no next Page
			return null;
		}
		presentPageNumber=new Integer((presentPageNumber.intValue()+1));
		Object[] details = (Object[])pagesWithNumber.get(presentPageNumber.toString());
		return (JPage)details[0];
	}

    /**
     *  Returns the previous page .
     * @return JPage
     */
    public JPage getPreviousPage() {
		if(presentPageNumber.intValue()==0 )
		{
			//There is no previous Page
			return null;
		}
		presentPageNumber=new Integer((presentPageNumber.intValue()-1));
		Object[] details = (Object[])pagesWithNumber.get(presentPageNumber.toString());
		return(JPage)details[0];
	}

    /**
     * Find page by it's name.
     * @param name String
     * @return JPage.
     */
    JPage getPageByName(String name)
	{
		
		Object[] details = (Object[])pageswithName.get(name);
		return (JPage)details[0];
	}

    /**
     *  Returns the page with the provided page number.
     * @param number
     * @return JPage
     */
    public JPage getPageByNumber(int number)
	{
		Integer intValue = new Integer(number);
		Object[] details = (Object[])pagesWithNumber.get(intValue.toString());
		return (JPage)details[0];

	}

    /**
     *  This returns the present page displayed in the Wizard.
     * @return JPage
     */
    public JPage getPresentPage()
    {
    	return getPageByName(presentPage);
    }
    
    /**
     * Return the name of the page which is displayed in wizard.
     * @return String Page name
     */
    public String getPresentPageName()
    {
    	return presentPage;
    }
    /**
     * Adds the details of the page to hashMap and wizard.
     * @param panel
     * @param name
     * @return boolean wether the add operation was successful or not.
     */
    boolean add(JPage panel,String name)
	{
		if(panel ==null)
		{
			return false;
		}
		if(panel instanceof JPage)
		{
			if(pageCount>=0)
			{
				Integer previousPage = new Integer(pageCount);
				Object[] previousPageDetails = (Object[])pagesWithNumber.get(previousPage.toString());
				panel.setPreviousPageName((String)previousPageDetails[1]);
				
				JPage previousPageObject = (JPage)previousPageDetails[0];
				previousPageObject.setNextPageName(name);
				
			}else
			{
				panel.setPreviousPageName(null);
			}
			
			pageCount++;
			Integer presentValueToAdd = new Integer(pageCount);
			Object[] pagesWithNumberObj = new Object[2];
			pagesWithNumberObj[0]=panel;
			pagesWithNumberObj[1]=name;
			pagesWithNumber.put(presentValueToAdd.toString(),pagesWithNumberObj);
			
			Object[] pagesWithNameObj = new Object[2];
			pagesWithNameObj[0]=panel;
			pagesWithNameObj[1]=presentValueToAdd;
			pageswithName.put(name,pagesWithNameObj);
			
			
			
			return true;
		}
		else
		{
			return false;
		}
	}
}
